package edu.umd.dawn.common.jwt;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.time.temporal.ChronoUnit;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;

import edu.umd.dawn.common.entities.User;
import edu.umd.dawn.common.exceptions.BaseExceptions;
import edu.umd.dawn.common.exceptions.DawnException;
import lombok.Getter;

@Getter
public class JWTUtil {

    private Algorithm algorithm;
    private String accessSecret;
    private String jwtString;
    private DecodedJWT jwt;

    public JWTUtil(String accessSecret, String jwt) {
        this.accessSecret = accessSecret;
        this.jwtString = jwt;
        initAlgorithm();
        decode();
    }

    public JWTUtil(String accessSecret) {
        this.accessSecret = accessSecret;
        initAlgorithm();
    }

    private void initAlgorithm() {
        this.algorithm = Algorithm.HMAC256(accessSecret);
    }

    private void decode() {
        try {
            this.jwt = JWT.require(algorithm)
                    .acceptLeeway(1) // 1 sec for nbf and iat
                    .acceptExpiresAt(5) // 5 secs for exp
                    .withClaimPresence("id") // need id in the claim
                    .build()
                    .verify(jwtString);
        } catch (Exception e) {
            throw new DawnException(BaseExceptions.INVALID_JWT, e);
        }
    }

    private String create(Claims claims, int expMin) {
        try {
            String token = claims.fill(JWT.create()
            .withIssuer("dawn")).withExpiresAt(LocalDateTime.now().plus(expMin, ChronoUnit.MINUTES).toInstant(ZoneOffset.UTC)).sign(algorithm);
            return token;
        } catch (Exception e) {
            throw new DawnException(BaseExceptions.INVALID_JWT, e);
        }
    }

    public Claims getClaims() {
        return Claims.build(this);
    }

    public static JWTUtil parse(String accessSecret, String jwt) {
        return new JWTUtil(accessSecret, jwt);
    }

    public static String issue(String accessSecret, Claims claims) {
        JWTUtil util = new JWTUtil(accessSecret);
        return util.create(claims, 20); // eventually move 20 to be a variable
    }

    public static String issue(String accessSecret, Claims claims, int expTime) {
        JWTUtil util = new JWTUtil(accessSecret);
        return util.create(claims, expTime);
    }

    public static String issue(String accessSecret, User user, int expTime) {
        JWTUtil util = new JWTUtil(accessSecret);
        return util.create(Claims.fromUser(user), expTime);
    }
}
